/*******************************************************************************
*                                                                              *
*                                   Viewmol                                    *
*                                                                              *
*                             C E L L F O R M . C                              *
*                                                                              *
*                    Copyright (c) Joerg-R. Hill, May 1999                     *
*                                                                              *
********************************************************************************
*
* $Id: cellform.c,v 1.2 1999/05/24 01:24:56 jrh Exp $
* $Log: cellform.c,v $
* Revision 1.2  1999/05/24 01:24:56  jrh
* Release 2.2.1
*
* Revision 1.1  1999/02/07 21:45:28  jrh
* Initial revision
*
*
*/
#include<math.h>
#include<stdio.h>
#include<X11/Intrinsic.h>
#include<Xm/Xm.h>
#include<Xm/BulletinB.h>
#include<Xm/Form.h>
#include<Xm/Label.h>
#include<Xm/LabelG.h>
#include<Xm/PanedW.h>
#include<Xm/PushB.h>
#include<Xm/PushBG.h>
#include<Xm/RowColumn.h>
#include<Xm/Scale.h>
#include<Xm/Separator.h>
#include<Xm/Text.h>
#include<Xm/ToggleB.h>
#include "GLwMDrawA.h"
#include "viewmol.h"
#include "dialog.h"

void unitcellExit(Widget, caddr_t, XmPushButtonCallbackStruct *);
void setCellSliders(struct MOLECULE *);
void makeNewCell(Widget, struct SLIDER *, XmScrollBarCallbackStruct *);

extern void GetSlider(Widget, struct SLIDER *, XmScrollBarCallbackStruct *);
extern void drawMolecule(Widget, caddr_t, GLwDrawingAreaCallbackStruct *);
extern int redraw(int);
extern void expandCell(void);
extern void MapBox(Widget, caddr_t, XmToggleButtonCallbackStruct *);
extern Widget CreatePushButtonRow(Widget, struct PushButtonRow *, int);
extern Widget CreateToggleBox(Widget, struct PushButtonRow *, int, int,
                              int, int, int);
extern void changeBoolean(Widget, int *, GLwDrawingAreaCallbackStruct *);
extern Widget initShell(Widget, char *, Widget *, Widget *);
extern void setMenuItem(int, int);
extern void *getmem(size_t, size_t);
extern void fremem(void **);

extern struct WINDOW windows[];
extern struct MOLECULE *molecules;
extern Widget topShell;
extern int showUnitCell;

static Widget dialog, aWidget, bWidget, cWidget;
static struct MOLECULE *mol;
static int showUnitCell_save;
static double cellFactor_save[3];

void unitcellDialog(Widget widget, caddr_t dummy, XmAnyCallbackStruct *data)
{
  Widget form, board, togglebox, sep;
  short decimals;
  static struct SLIDER aSlider, bSlider, cSlider;
  static struct PushButtonRow checkbox_button[] = {{"visible", changeBoolean,
                                                    (XtPointer)&showUnitCell, NULL}};
  static struct PushButtonRow buttons[] = {
    { "ok", unitcellExit, (XtPointer)TRUE, NULL },
    { "cancel", unitcellExit, (XtPointer)FALSE, NULL },
  };

  /* This function creates the dialog for the "Unit cell" option */

  if (windows[VIEWER].set < 0)
    mol=&molecules[0];
  else
    mol=&molecules[windows[VIEWER].set];
  setMenuItem(VIEWER_UNITCELL, False);
  showUnitCell_save=showUnitCell;
  cellFactor_save[0]=mol->cellFactor[0];
  cellFactor_save[1]=mol->cellFactor[1];
  cellFactor_save[2]=mol->cellFactor[2];

  dialog=initShell(XtParent(windows[VIEWER].widget), "unitcellForm", &board,
			 &form);
  togglebox=CreateToggleBox(form, checkbox_button, XtNumber(checkbox_button),
                            XmVERTICAL, 1, False, showUnitCell);
  XtVaSetValues(togglebox, XmNleftAttachment, XmATTACH_FORM,
                           XmNrightAttachment, XmATTACH_FORM,
                           XmNtopAttachment, XmATTACH_FORM,
                           NULL);

  sep=XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass, form,
                              XmNleftAttachment, XmATTACH_FORM,
                              XmNrightAttachment, XmATTACH_FORM,
                              XmNtopAttachment, XmATTACH_WIDGET,
                              XmNtopWidget, togglebox,
                              NULL);
  aWidget=XtVaCreateManagedWidget("avalue", xmScaleWidgetClass, form,
                                  XmNorientation, XmHORIZONTAL,
                                  XmNshowValue, True,
                                  XmNsensitive, True,
                                  NULL);
  XtVaGetValues(aWidget, XmNdecimalPoints, &decimals, NULL);
  XtVaSetValues(aWidget, XmNvalue, (int)(mol->cellFactor[0]*pow(10.0,
		    (double)decimals)), NULL);
  aSlider.number=&(mol->cellFactor[0]);
  aSlider.decimals=decimals;
  aSlider.draw=(void (*)())drawMolecule;
  XtAddCallback(aWidget, XmNvalueChangedCallback, (XtCallbackProc)makeNewCell,
		    &aSlider);
  bWidget=XtVaCreateManagedWidget("bvalue", xmScaleWidgetClass, form,
                                  XmNorientation, XmHORIZONTAL,
                                  XmNshowValue, True,
                                  XmNsensitive, True,
                                  NULL);
  XtVaGetValues(bWidget, XmNdecimalPoints, &decimals, NULL);
  XtVaSetValues(bWidget, XmNvalue, (int)(mol->cellFactor[1]*pow(10.0,
		    (double)decimals)), NULL);
  bSlider.number=&(mol->cellFactor[1]);
  bSlider.decimals=decimals;
  bSlider.draw=(void (*)())drawMolecule;
  XtAddCallback(bWidget, XmNvalueChangedCallback, (XtCallbackProc)makeNewCell,
		    &bSlider);
  cWidget=XtVaCreateManagedWidget("cvalue", xmScaleWidgetClass, form,
                                  XmNorientation, XmHORIZONTAL,
                                  XmNshowValue, True,
                                  XmNsensitive, True,
                                  NULL);
  XtVaGetValues(cWidget, XmNdecimalPoints, &decimals, NULL);
  XtVaSetValues(cWidget, XmNvalue, (int)(mol->cellFactor[2]*pow(10.0,
		    (double)decimals)), NULL);
  cSlider.number=&(mol->cellFactor[2]);
  cSlider.decimals=decimals;
  cSlider.draw=(void (*)())drawMolecule;
  XtAddCallback(cWidget, XmNvalueChangedCallback, (XtCallbackProc)makeNewCell,
		    &cSlider);
  sep=XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass, form,
                              XmNleftAttachment, XmATTACH_FORM,
                              XmNrightAttachment, XmATTACH_FORM,
                              XmNtopAttachment, XmATTACH_WIDGET,
                              XmNtopWidget, form,
                              NULL);

  CreatePushButtonRow(form, buttons, XtNumber(buttons));
  XtAddCallback(dialog, XmNpopupCallback, (XtCallbackProc)MapBox,
                (XmAnyCallbackStruct *)NULL);
  XtManageChild(form);
  XtManageChild(board);
}

void unitcellExit(Widget button, caddr_t which,
			XmPushButtonCallbackStruct *data)
{
  XtDestroyWidget(dialog);
  dialog=NULL;
  if (!(int)which)
  {
    showUnitCell=showUnitCell_save;
    if (mol->cellFactor[0] != cellFactor_save[0] ||
	  mol->cellFactor[1] != cellFactor_save[1] ||
        mol->cellFactor[2] != cellFactor_save[2])
    {
      mol->cellFactor[0]=cellFactor_save[0];
      mol->cellFactor[1]=cellFactor_save[1];
      mol->cellFactor[2]=cellFactor_save[2];
      expandCell();
    }
    redraw(VIEWER);
  }
  setMenuItem(VIEWER_UNITCELL, True);
}

void setCellSliders(struct MOLECULE *mol)
{
  short decimals;

  if (dialog)
  {
    XtVaGetValues(aWidget, XmNdecimalPoints, &decimals, NULL);
    XtVaSetValues(aWidget, XmNvalue, (int)(mol->cellFactor[0]*pow(10.0,
                  (double)decimals)), NULL);
    XtVaGetValues(bWidget, XmNdecimalPoints, &decimals, NULL);
    XtVaSetValues(bWidget, XmNvalue, (int)(mol->cellFactor[1]*pow(10.0,
                  (double)decimals)), NULL);
    XtVaGetValues(cWidget, XmNdecimalPoints, &decimals, NULL);
    XtVaSetValues(cWidget, XmNvalue, (int)(mol->cellFactor[2]*pow(10.0,
                  (double)decimals)), NULL);
  }
}

void makeNewCell(Widget w, struct SLIDER *slider, XmScrollBarCallbackStruct *data)
{
  GetSlider(w, slider, data);
  if (mol->cellFactor[0] != cellFactor_save[0] || mol->cellFactor[1] != cellFactor_save[1] ||
      mol->cellFactor[2] != cellFactor_save[2]) expandCell();
  redraw(VIEWER);
}
