/*******************************************************************************
*                                                                              *
*                                   Viewmol                                    *
*                                                                              *
*                               T H E R M O . C                                *
*                                                                              *
*                    Copyright (c) Joerg-R. Hill, May 1999                     *
*                                                                              *
********************************************************************************
*
* $Id: thermo.c,v 1.1 1999/05/24 01:29:53 jrh Exp $
* $Log: thermo.c,v $
* Revision 1.1  1999/05/24 01:29:53  jrh
* Initial revision
*
*/
#include<math.h>
#include<stdio.h>
#include "viewmol.h"

void thermo(struct MOLECULE *);
int  isLinear(struct MOLECULE *);
int getSymmetryNumber(struct MOLECULE *);

extern struct WINDOW windows[];
extern struct MOLECULE *molecules;
extern double temp;
extern int debug;

void thermo(struct MOLECULE *mol)
{
  const double N=6.022169e23, k=1.380622e-23;
  const double h=6.626196e-34, c=2.997925e10;
  double R=k*N, pi=4.0*atan(1.0), RT=R*temp;
  double pressure=1.0;
  double ctrl, htrl, strl, crot, hrot, srot, cvib, hvib, svib;
  double mass, hc, u, r, s;
  int sigma;
  register int i;

  /* Translation */
  htrl=2.5*RT;
  ctrl=2.5*R;
  mass=0.0;
  for (i=0; i<mol->na; i++)
    mass+=mol->atoms[i].mass;
  strl=R*(2.5+1.5*log(2.0*pi*k/(h*h))+log(k)+1.5*log(mass/N)+2.5*log(temp)-log(pressure));

  /* Rotation */
  sigma=getSymmetryNumber(mol);
  if (isLinear(mol))
  {
    hrot=RT;
    crot=R;
    srot=R*log(k*temp/((double)(sigma)*h*mol->rotConstants[0]))+R;
  }
  else
  {
    hrot=1.5*RT;
    crot=1.5*R;
    srot=0.5*R*log(pi*k*k*k*temp*temp*temp/(sqrt((double)sigma)*h*h*h*c*c*c*mol->rotConstants[0]*mol->rotConstants[1]*mol->rotConstants[2]))+1.5*R;
  }

  /* Vibration */
  cvib=0.0;
  hvib=0.0;
  svib=0.0;
  if (mol->na > 1)
  {
    hc=h*c;
    for (i=0; i<mol->nmodes; i++)
    {
	if (mol->normal_modes[i].wavenumber < 0.0) continue;
	u=hc*mol->normal_modes[i].wavenumber/(k*temp);
	r=exp(-u);
	s=1.0/(1.0-r);
	cvib+=u*u*r*s*s;
	hvib+=hc*mol->normal_modes[i].wavenumber*(r*s+0.5);
	svib+=u*r*s-log(1.0/s);
    }
    cvib*=R;
    hvib*=N;
    svib*=R;
  }
  if (debug)
  {
    printf("              C/[J/mol]    H/[J/mol]  S[J/(mol K)]\n");
    printf("Translation %12.6f %12.6f %12.6f\n", ctrl, htrl, strl);
    printf("Rotation    %12.6f %12.6f %12.6f\n", crot, hrot, srot);
    printf("Vibration   %12.6f %12.6f %12.6f\n", cvib, hvib, svib);
  }
}

int isLinear(struct MOLECULE *mol)
{
  if (mol->rotConstants[0] == 0.0 || mol->rotConstants[1] == 0.0 ||
	mol->rotConstants[2] == 0.0)
    return(TRUE);
  else
    return(FALSE);
}

int getSymmetryNumber(struct MOLECULE *mol)
{
  return(1);
}
